/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.jbi.management.admin;

import java.util.List;
import java.util.Map;

import javax.jbi.management.ComponentLifeCycleMBean;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.ow2.petals.container.lifecycle.Installer;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle;
import org.ow2.petals.jbi.management.MBeanNamesImpl;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.w3c.dom.Document;

/**
 * Internal admin methods.
 * 
 * @author rnaudin
 * 
 */
public interface AdminService extends AdminServiceMBean {

    /**
     * Get the PEtALS {@code MbeanNames} instance.
     * 
     * @return the instance
     */
    MBeanNamesImpl getMBeanNames();

    /**
     * Get the {@code MBeanServer}.
     * 
     * @return the instance
     */
    MBeanServer getMBeanServer();

    /**
     * Get the service description for a endpoint of a component.
     * 
     * @param componentName
     *            name of the component
     * @param endpoint
     *            endpoint
     * @return DOM Document containing the service description. Can be null if
     *         not retrieved from the component
     */
    Document getServiceDescription(String componentName, ServiceEndpoint endpoint)
            throws PetalsException;

    /**
     * Get the {@link Installer} by its component name.
     * 
     * @param name
     * @return the {@link Installer}
     */
    Installer getInstallerByName(String name);

    /**
     * Get the {@link ObjectName} by its installer name.
     * 
     * @param name
     * @return the {@link ObjectName}
     */
    ObjectName getInstallerMBeanNameByName(String name);

    /**
     * Get the list of {@link Installer}s
     * 
     * @return
     */
    Installer[] getInstallers();

    /**
     * Find the {@link ComponentLifeCycleMBean} of a JBI Component by its unique
     * name.
     * 
     * @param name
     *            the name of the binding component; must be non-null and
     *            non-empty
     * @return the {@link ComponentLifeCycleMBean} of the component, or
     *         <code>null</code> if there is no such component with the given
     *         <code>name</code>
     */
    ComponentLifeCycleMBean getBindingComponentLifeCycleByName(String name);

    /**
     * Find the {@link ComponentLifeCycleMBean} of a JBI Component by its unique
     * name.
     * 
     * @param name
     *            the name of the engine component; must be non-null and
     *            non-empty
     * @return the {@link ComponentLifeCycleMBean} of the component, or
     *         <code>null</code> if there is no such component with the given
     *         <code>name</code>
     */
    ComponentLifeCycleMBean getEngineComponentLifeCycleByName(String name);

    /**
     * Get the {@link ServiceAssemblyLifeCycle} by its unique name.
     * 
     * @param name
     * @return the {@link ServiceAssemblyLifeCycle}
     */
    ServiceAssemblyLifeCycle getServiceAssemblyByName(String saName);

    /**
     * Get the map of {@link ServiceAssemblyLifeCycle} by names.
     * 
     * @return the map
     */
    Map<String, ServiceAssemblyLifeCycle> getServiceAssemblies();

    /**
     * Get the {@link SharedLibraryLifeCycle} by its unique name.
     * 
     * @param name
     * @return the {@link SharedLibraryLifeCycle}
     */
    SharedLibraryLifeCycle getSharedLibraryByName(String slName);
    
    /**
     * Get the map of {@link SharedLibraryLifeCycle} by names.
     * 
     * @return the map
     */
    Map<String, SharedLibraryLifeCycle> getSharedLibraries();

    List<ServiceUnitLifeCycle> getServiceUnitsLifeCyclesForComponent(String component);

    ObjectName registerBindingComponent(ComponentLifeCycleMBean lifeCycle, String name)
            throws ManagementException;

    ObjectName registerEngineComponent(ComponentLifeCycleMBean lifeCycle, String name)
            throws ManagementException;

    ObjectName registerInstaller(Installer installerMBean, String name) throws ManagementException;

    void registerServiceAssembly(ServiceAssemblyLifeCycle serviceAssemblyLifeCycle, String name);

    void registerSharedLibrary(SharedLibraryLifeCycle sharedLibraryLifeCycle, String name);

    void unregisterBindingComponent(String name) throws ManagementException;

    void unregisterEngineComponent(String name) throws ManagementException;

    void unregisterInstaller(String name) throws ManagementException;

    void unregisterServiceAssembly(String saName) throws ManagementException;

    void unregisterSharedLibrary(String slName) throws ManagementException;

}
