/**
 * PETALS: PETALS Services Platform Copyright (C) 2005 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.admin;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.JBIException;
import javax.jbi.management.ComponentLifeCycleMBean;
import javax.jbi.management.InstallerMBean;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.jmx.JMXService;
import org.ow2.petals.container.lifecycle.Installer;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle;
import org.ow2.petals.jbi.management.MBeanNamesImpl;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.server.MBeanHelper;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.ParameterCheckHelper;
import org.w3c.dom.Document;

/**
 * TODO AdminServiceImpl : check the exceptions catch
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.management.admin.AdminService.class))
public class AdminServiceImpl implements AdminService {

    /**
     * Logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog
     */
    @Monolog(name = "logger")
    private Logger logger;

    /**
     * The JMX Fractal service
     */
    @Requires(name = "jmx", signature = org.ow2.petals.communication.jmx.JMXService.class)
    private JMXService jmxService;

    private Map<String, Installer> installersByName;

    private Map<ObjectName, ComponentLifeCycleMBean> bindingLifeCyclesByMBeanName;

    private Map<ObjectName, ComponentLifeCycleMBean> engineLifeCyclesByMBeanName;

    private Map<String, ComponentLifeCycleMBean> bindingLifeCyclesByName;

    private Map<String, ComponentLifeCycleMBean> engineLifeCyclesByName;

    private Map<String, SharedLibraryLifeCycle> sharedLibraryLifeCycles;

    private Map<String, ServiceAssemblyLifeCycle> serviceAssemblyLifeCycles;

    private MBeanNamesImpl mBeanNamesImpl;

    private MBeanServer mbeanServer;

    /**
     * Default constructor
     */
    public AdminServiceImpl() {
        super();
    }

    /*
     * @see javax.jbi.management.AdminServiceMBean#getBindingComponents()
     */
    public ObjectName[] getBindingComponents() {
        log.call();

        ObjectName[] names = new ObjectName[0];
        names = bindingLifeCyclesByMBeanName.keySet().toArray(names);

        return names;
    }

    /*
     * @see javax.jbi.management.AdminServiceMBean#getEngineComponents()
     */
    public ObjectName[] getEngineComponents() {
        log.call();

        ObjectName[] names = new ObjectName[0];
        names = engineLifeCyclesByMBeanName.keySet().toArray(names);

        return names;
    }

    /*
     * @see javax.jbi.management.AdminServiceMBean#getComponentByName(java.lang.
     * String)
     */
    public ObjectName getComponentByName(String name) {
        log.call();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(name, "name must not be null", log);

        ObjectName result = null;

        for (ObjectName mbeanName : bindingLifeCyclesByMBeanName.keySet()) {
            if (name.equals(mBeanNamesImpl.getNameFromMBeanName(mbeanName))) {
                result = mbeanName;
                break;
            }
        }
        if (result == null) {
            for (ObjectName mbeanName : engineLifeCyclesByMBeanName.keySet()) {
                if (name.equals(mBeanNamesImpl.getNameFromMBeanName(mbeanName))) {
                    result = mbeanName;
                    break;
                }
            }
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.AdminServiceMBean#getSystemInfo()
     */
    public String getSystemInfo() {
        return "Petals JBI Container - version: "
                + Package.getPackage("org.ow2.petals").getImplementationVersion();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.AdminServiceMBean#getSystemService(java.lang.String)
     */
    public ObjectName getSystemService(String serviceName) {
        log.call();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceName,
                "serviceName must not be null", log);

        ObjectName result = null;
        ObjectName[] systemServices = getSystemServices();

        for (ObjectName objectName : systemServices) {
            if (objectName.getCanonicalName().equals(serviceName)) {
                result = objectName;
                break;
            }
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.AdminServiceMBean#getSystemServices()
     */
    public ObjectName[] getSystemServices() {
        log.call();

        ObjectName[] names = new ObjectName[3];

        try {
            names[0] = MBeanHelper.retrieveServiceMBean(MBeanHelper.ADMIN_MBEAN, this.mbeanServer);
            names[1] = MBeanHelper.retrieveServiceMBean(MBeanHelper.DEPLOYMENT_MBEAN,
                    this.mbeanServer);
            names[2] = MBeanHelper.retrieveServiceMBean(MBeanHelper.INSTALLATION_MBEAN,
                    this.mbeanServer);
        } catch (MalformedObjectNameException e) {
            log.warning("A system service name can't be retrieved !", e);
        }

        return names;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.AdminServiceMBean#isBinding(java.lang.String)
     */
    public boolean isBinding(String componentName) {
        ParameterCheckHelper.isNullOrEmptyParameterWithLog(componentName,
                "componentName must not be null", log);

        return this.bindingLifeCyclesByName.containsKey(componentName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.AdminServiceMBean#isEngine(java.lang.String)
     */
    public boolean isEngine(String componentName) {
        ParameterCheckHelper.isNullOrEmptyParameterWithLog(componentName,
                "componentName must not be null", log);

        return this.engineLifeCyclesByName.containsKey(componentName);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminServiceMBean#
     * isExchangeWithConsumerOkayForComponent(java.lang.String,
     * javax.jbi.servicedesc.ServiceEndpoint,
     * javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithConsumerOkayForComponent(String componentName,
            ServiceEndpoint internalEndpoint, MessageExchange exchange) throws JBIException {
        log.start();

        Installer installer = this.installersByName.get(componentName);
        if (installer == null) {
            throw new JBIException("Cannot find a component '" + componentName
                    + "' in the container");
        }

        boolean result = installer.getComponent().isExchangeWithConsumerOkay(internalEndpoint,
                exchange);

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminServiceMBean#
     * isExchangeWithProviderOkayForComponent(java.lang.String,
     * javax.jbi.servicedesc.ServiceEndpoint,
     * javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithProviderOkayForComponent(String componentName,
            ServiceEndpoint internalEndpoint, MessageExchange exchange) throws JBIException {
        log.start();

        Installer installer = this.installersByName.get(componentName);
        if (installer == null) {
            throw new JBIException("Cannot find a component '" + componentName
                    + "' in the container");
        }

        boolean result = installer.getComponent().isExchangeWithProviderOkay(internalEndpoint,
                exchange);

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.admin.AdminService#getMBeanNames()
     */
    public MBeanNamesImpl getMBeanNames() {
        return this.mBeanNamesImpl;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.admin.AdminService#getMBeanServer()
     */
    public MBeanServer getMBeanServer() {
        return this.mbeanServer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getServiceDescription
     * (java.lang.String, javax.jbi.servicedesc.ServiceEndpoint)
     */
    public Document getServiceDescription(String componentName, ServiceEndpoint endpoint)
            throws PetalsException {
        log.start();

        Installer installer = this.installersByName.get(componentName);
        if (installer == null) {
            throw new PetalsException("Cannot find a component '" + componentName
                    + "' in the container");
        }

        Document doc = installer.getComponent().getServiceDescription(endpoint);

        log.end();
        return doc;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminService#
     * getBindingComponentLifeCycleByName(java.lang.String)
     */
    public ComponentLifeCycleMBean getBindingComponentLifeCycleByName(String name) {
        ParameterCheckHelper.isNullOrEmptyParameterWithLog(name, "name must not be null or empty",
                log);
        log.call();

        return this.bindingLifeCyclesByName.get(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminService#
     * getEngineComponentLifeCycleByName(java.lang.String)
     */
    public ComponentLifeCycleMBean getEngineComponentLifeCycleByName(String name) {
        log.call();

        return this.engineLifeCyclesByName.get(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getInstallerByName(java
     * .lang.String)
     */
    public Installer getInstallerByName(String name) {
        return this.installersByName.get(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getInstallerMBeanNameByName
     * (java.lang.String)
     */
    public ObjectName getInstallerMBeanNameByName(String name) {
        return this.mBeanNamesImpl.createInstallerMBeanName(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.admin.AdminService#getInstallers()
     */
    public Installer[] getInstallers() {
        return this.installersByName.values().toArray(new Installer[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getServiceAssemblyByName
     * (java.lang.String)
     */
    public ServiceAssemblyLifeCycle getServiceAssemblyByName(String saName) {
        return this.serviceAssemblyLifeCycles.get(saName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getServiceAssemblies()
     */
    public Map<String, ServiceAssemblyLifeCycle> getServiceAssemblies() {
        return this.serviceAssemblyLifeCycles;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getSharedLibraryByName
     * (java.lang.String)
     */
    public SharedLibraryLifeCycle getSharedLibraryByName(String slName) {
        return this.sharedLibraryLifeCycles.get(slName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getSharedLibraries()
     */
    public Map<String, SharedLibraryLifeCycle> getSharedLibraries() {
        return this.sharedLibraryLifeCycles;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminService#
     * getServiceUnitsLifeCyclesForComponent(java.lang.String)
     */
    public List<ServiceUnitLifeCycle> getServiceUnitsLifeCyclesForComponent(String component) {
        this.log.call();

        List<ServiceUnitLifeCycle> serviceUnitLifeCycles = new ArrayList<ServiceUnitLifeCycle>();

        for (ServiceAssemblyLifeCycle serviceAssemblyLifeCycle : this.serviceAssemblyLifeCycles
                .values()) {
            for (ServiceUnitLifeCycle serviceUnitLifeCycle : serviceAssemblyLifeCycle
                    .getServiceUnitLifeCycles()) {
                if (serviceUnitLifeCycle.getTargetComponentName().equals(component)) {
                    serviceUnitLifeCycles.add(serviceUnitLifeCycle);
                }
            }
        }

        return serviceUnitLifeCycles;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerBindingComponent
     * (javax.jbi.management.ComponentLifeCycleMBean, java.lang.String)
     */
    public ObjectName registerBindingComponent(ComponentLifeCycleMBean lifeCycle, String name)
            throws ManagementException {
        this.log.call();

        ObjectName mbeanName = mBeanNamesImpl.createBindingComponentMBeanName(name);
        try {
            this.mbeanServer.registerMBean(MBeanHelper.createCustomMBean(lifeCycle,
                    ComponentLifeCycleMBean.class), mbeanName);
            this.bindingLifeCyclesByMBeanName.put(mbeanName, lifeCycle);
            this.bindingLifeCyclesByName.put(name, lifeCycle);
        } catch (Exception e) {
            throw new ManagementException(e);
        }

        return mbeanName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerEngineComponent
     * (javax.jbi.management.ComponentLifeCycleMBean, java.lang.String)
     */
    public ObjectName registerEngineComponent(ComponentLifeCycleMBean lifeCycle, String name)
            throws ManagementException {
        this.log.call();

        ObjectName mbeanName = mBeanNamesImpl.createEngineComponentMBeanName(name);
        try {
            this.mbeanServer.registerMBean(MBeanHelper.createCustomMBean(lifeCycle,
                    ComponentLifeCycleMBean.class), mbeanName);
            this.engineLifeCyclesByMBeanName.put(mbeanName, lifeCycle);
            this.engineLifeCyclesByName.put(name, lifeCycle);
        } catch (Exception e) {
            throw new ManagementException(e);
        }

        return mbeanName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerInstaller(org
     * .ow2.petals.container.lifecycle.Installer, java.lang.String)
     */
    public ObjectName registerInstaller(Installer installer, String name)
            throws ManagementException {
        this.log.call();

        ObjectName mbeanName = mBeanNamesImpl.createInstallerMBeanName(name);
        try {
            this.mbeanServer.registerMBean(MBeanHelper.createCustomMBean(installer,
                    InstallerMBean.class), mbeanName);
            this.installersByName.put(name, installer);
        } catch (Exception e) {
            throw new ManagementException(e);
        }

        return mbeanName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerServiceAssembly
     * (org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle,
     * java.lang.String)
     */
    public void registerServiceAssembly(ServiceAssemblyLifeCycle serviceAssemblyLifeCycle,
            String name) {
        this.log.call();

        this.serviceAssemblyLifeCycles.put(name, serviceAssemblyLifeCycle);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerSharedLibrary
     * (org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle,
     * java.lang.String)
     */
    public void registerSharedLibrary(SharedLibraryLifeCycle sharedLibraryLifeCycle, String name) {
        this.sharedLibraryLifeCycles.put(name, sharedLibraryLifeCycle);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterBindingComponent
     * (java.lang.String)
     */
    public void unregisterBindingComponent(String name) throws ManagementException {
        this.log.call();

        ObjectName objectName = this.mBeanNamesImpl.createBindingComponentMBeanName(name);
        try {
            this.mbeanServer.unregisterMBean(objectName);
        } catch (InstanceNotFoundException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        } catch (MBeanRegistrationException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        }

        this.bindingLifeCyclesByMBeanName.remove(objectName);
        this.bindingLifeCyclesByName.remove(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterEngineComponent
     * (java.lang.String)
     */
    public void unregisterEngineComponent(String name) throws ManagementException {
        this.log.call();

        ObjectName objectName = this.mBeanNamesImpl.createEngineComponentMBeanName(name);
        try {
            this.mbeanServer.unregisterMBean(objectName);
        } catch (InstanceNotFoundException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        } catch (MBeanRegistrationException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        }

        this.engineLifeCyclesByMBeanName.remove(objectName);
        this.engineLifeCyclesByName.remove(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterInstaller(
     * java.lang.String)
     */
    public void unregisterInstaller(String name) throws ManagementException {
        ObjectName objectName = this.mBeanNamesImpl.createInstallerMBeanName(name);
        try {
            this.mbeanServer.unregisterMBean(objectName);
        } catch (InstanceNotFoundException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        } catch (MBeanRegistrationException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        }

        this.installersByName.remove(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterServiceAssembly
     * (java.lang.String)
     */
    public void unregisterServiceAssembly(String name) throws ManagementException {
        if (this.serviceAssemblyLifeCycles.remove(name) == null) {
            throw new ManagementException("Failed to unregistered the Service Assembly '" + name
                    + "'");
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterSharedLibrary
     * (java.lang.String)
     */
    public void unregisterSharedLibrary(String name) throws ManagementException {
        if (this.sharedLibraryLifeCycles.remove(name) == null) {
            throw new ManagementException("Failed to unregistered the Shared Library '" + name
                    + "'");
        }
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IllegalLifeCycleException {
        log = new LoggingUtil(logger);
        log.start();

        this.mbeanServer = jmxService.getLocalJMXServer();

        this.mBeanNamesImpl = new MBeanNamesImpl("org.ow2.petals");

        this.bindingLifeCyclesByMBeanName = new ConcurrentHashMap<ObjectName, ComponentLifeCycleMBean>();
        this.engineLifeCyclesByMBeanName = new ConcurrentHashMap<ObjectName, ComponentLifeCycleMBean>();

        this.bindingLifeCyclesByName = new ConcurrentHashMap<String, ComponentLifeCycleMBean>();
        this.engineLifeCyclesByName = new ConcurrentHashMap<String, ComponentLifeCycleMBean>();

        this.serviceAssemblyLifeCycles = new ConcurrentHashMap<String, ServiceAssemblyLifeCycle>();

        this.installersByName = new ConcurrentHashMap<String, Installer>();

        this.sharedLibraryLifeCycles = Collections
                .synchronizedMap(new HashMap<String, SharedLibraryLifeCycle>());

        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws IllegalLifeCycleException {
        this.log.call();

    }

}
