/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.jbi.management.installation;

import java.net.URL;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.jmx.JMXService;
import org.ow2.petals.util.LoggingUtil;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.management.installation.ComponentInstallationService.class))
public class ComponentInstallationServiceImpl implements ComponentInstallationService {

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    @Requires(name = "installation", signature = InstallationServiceMBean.class)
    private InstallationServiceMBean installationService;

    @Requires(name = "jmx", signature = JMXService.class)
    private JMXService jmxService;

    @LifeCycle(on = LifeCycleType.START)
    public void start() {
        this.log = new LoggingUtil(this.logger);
    }

    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
    }

    public boolean install(URL componentZipURL) {
        this.log.debug("Installing component from " + componentZipURL);
        boolean result = true;
        try {
            MBeanServer jmxServer = jmxService.getLocalJMXServer();

            ObjectName newInstallerName = installationService.loadNewInstaller(componentZipURL
                    .toString());
            if (!newInstallerName.getDomain().equals("error")) {
                Object[] nullObjects = new Object[0];
                String[] nullStrings = new String[0];
                Object o = jmxServer.invoke(newInstallerName, "install", nullObjects, nullStrings);
                if (o instanceof ObjectName) {
                    ObjectName componentName = (ObjectName) o;
                    jmxServer.invoke(componentName, "start", nullObjects, nullStrings);
                } else {
                    throw new Exception(
                            "The install() invokation on the component installer has returned an unexpected object: "
                                    + result);
                }
            } else {
                throw new Exception("Component can't be installed");
            }
        } catch (Throwable e) {
            log.error("Error occured during auto installation: ", e);
            result = false;
        }
        return result;
    }

    public boolean uninstall(String componentId) {
        this.log.debug("Uninstall component " + componentId);
        return installationService.forceUnloadComponentInstaller(componentId);
    }
}
