/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.util.HashMap;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task checks the validity of a package to install. <br/>
 * Inputs : unzipped package root {@link File} <br/>
 * Outputs : Entity name (String), {@link Jbi} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre
 * @author Frederic Gardes - eBM Websourcing
 */
public abstract class AbstractCheckPackageTask extends AbstractLoggableTask {

    private PackageHelper packageHandler;

    public AbstractCheckPackageTask(LoggingUtil log, PackageHelper packageHandler) {
        super(log);
        this.packageHandler = packageHandler;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {

        File packageFile = (File) context.get(TaskContextConstants.UNZIP_ROOT);
        /*
         * Load descriptor
         */
        Jbi descriptor = packageHandler.loadDescriptor(packageFile);
        /*
         * Check the entity package
         */
        checkPackage(descriptor, packageFile);
        /*
         * Extract entity unique identifier
         */
        context.put(TaskContextConstants.DESCRIPTOR, descriptor);
        context.put(TaskContextConstants.ENTITY_NAME, extractEntityName(descriptor));

    }

    protected abstract String extractEntityName(Jbi descriptor);

    protected abstract void checkPackage(Jbi descriptor, File packageFile) throws PetalsException;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        // Nothing to do
    }
}
