/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: RetrieveSALifeCycleTask.java 154 10 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.undeploy;

import java.util.HashMap;

import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.ow2.petals.processor.Task;

/**
 * Retrieve service assembly lifecycle.<br/>
 * Inputs : entity name {@link String} <br/>
 * Outputs : {@link ServiceAssemblyLifeCycle}<br />
 * The task has got a robust approach: it tries to do all the subtasks<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class RetrieveSALifeCycleTask implements Task {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminService adminService;

    public RetrieveSALifeCycleTask(AdminService adminService) {
        super();
        this.adminService = adminService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        if (context.containsKey(TaskContextConstants.ENTITY_NAME)) {
            String name = (String) context.get(TaskContextConstants.ENTITY_NAME);
            if (name != null) {
                ServiceAssemblyLifeCycle saLifeCycle = retrieveServiceAssemblyLifeCycle(name);
                if (saLifeCycle != null) {
                    /*
                     * Fill context
                     */
                    context.put(DeploymentContextConstants.SA_LIFECYCLE, saLifeCycle);
                }
            }
        }
    }

    /**
     * Retrieve the {@link ServiceAssemblyLifeCycle} matching the given service
     * assembly name.
     * 
     * @param saName
     *            the service assembly name
     * @return the retrieved {@link ServiceAssemblyLifeCycle}
     * @throws Exception
     *             if no service assembly can be retrieved
     */
    protected ServiceAssemblyLifeCycle retrieveServiceAssemblyLifeCycle(String saName) {
        ServiceAssemblyLifeCycle saLifeCycle = adminService.getServiceAssemblyByName(saName);
        return saLifeCycle;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        context.remove(DeploymentContextConstants.SA_LIFECYCLE);
    }

}
