/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.install.component;

import java.io.File;

import org.ow2.petals.jbi.descriptor.original.generated.Component;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.AbstractCheckPackageTask;
import org.ow2.petals.jbi.management.task.installation.InstallationUtils;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.StringHelper;

/**
 * This task checks the validity of the component package.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class ComponentCheckPackageTask extends AbstractCheckPackageTask {

    protected AdminService adminService;

    public ComponentCheckPackageTask(LoggingUtil log, PackageHelper packageHandler,
            AdminService adminService) {
        super(log, packageHandler);
        this.adminService = adminService;
    }

    /**
     * Check the validity of the component archive
     * 
     * @param descriptor
     *            the component <code>Jbi</code>
     * @param filePackage
     *            the service assembly package root
     * @return true if checking is ok and false otherwise
     * @throws PetalsException
     *             if component element or name is null or empty, if component
     *             is already installed or if some classpath elements of the
     *             component are missing
     */
    @Override
    protected void checkPackage(Jbi descriptor, File filePackage) throws PetalsException {
        /*
         * check if JBI descriptor has a component element
         */
        Component comp = descriptor.getComponent();
        if (comp == null) {
            throw new PetalsException("The component production element was not found in "
                    + "JBI Descriptor within installation package: "
                    + filePackage.getAbsolutePath());
        } else {
            /*
             * check if it is already loaded by package uuid
             */
            String compName = comp.getIdentification().getName();
            if (StringHelper.isNullOrEmpty(compName)) {
                throw new PetalsException(
                        "The component name must be non null and non empty for the new component :"
                                + filePackage.getAbsolutePath());
            }
            if (adminService.getInstallerByName(compName) != null) {
                throw new PetalsException("Other installation package already exists loaded in "
                        + "the component framework: " + compName);
            } else {
                /*
                 * Check the classpath elements of the component
                 */
                InstallationUtils.checkClassPathElements(filePackage, comp.getBootstrapClassPath()
                        .getPathElement());
                InstallationUtils.checkClassPathElements(filePackage, comp.getComponentClassPath()
                        .getPathElement());
            }
        }
    }

    @Override
    protected String extractEntityName(Jbi descriptor) {
        return descriptor.getComponent().getIdentification().getName();
    }

}
