/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.uninstall.component;

import java.util.HashMap;

import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.installation.InstallationContextConstants;
import org.ow2.petals.platform.systemstate.ComponentState;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task removes the component from the state holder file<br/>
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * Inputs : component name <br/>
 * Outputs : {@link ComponentState}
 * 
 * @author ofabre
 * @author Frederic Gardes - eBM Websourcing
 */
public class RemoveComponentStateHolderTask extends AbstractLoggableTask {

    private SystemStateService systemStateService = null;

    public RemoveComponentStateHolderTask(LoggingUtil log, SystemStateService systemStateService) {
        super(log);
        this.systemStateService = systemStateService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        if (context.containsKey(TaskContextConstants.ENTITY_NAME)) {
            String componentName = (String) context.get(TaskContextConstants.ENTITY_NAME);
            try {
                ComponentState componentState = removeComponentState(componentName);
                if (componentState != null) {
                    context.put(InstallationContextConstants.COMPONENT_STATE, componentState);
                }
            } catch (Exception e) {
                log.error("The component '" + componentName + "' state holder can not be removed",
                        e);
            }
        }
    }

    /**
     * Remove component from state holder file
     * 
     * @param componentName
     * @return the ComponentState extracted from component state file before
     *         deletion
     * @throws Exception
     */
    private ComponentState removeComponentState(String componentName) throws Exception {
        ComponentState componentState = systemStateService
                .deleteComponentStateHolder(componentName);
        return componentState;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        // TODO : recreate the state holder
    }

}
