/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.transport.util;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.ow2.petals.jbi.messaging.exchange.DeliveryChannelImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;

/**
 * @author garciap
 * 
 */
public final class TransporterUtil {

    public static final String PROPERTY_SENDSYNC_PROVIDER = "javax.jbi.messaging.sendSync.provider";

    public static final String PROPERTY_SENDSYNC_CONSUMER = "javax.jbi.messaging.sendSync.consumer";

    public static final String PROPERTY_OVERSENDSYNC = "javax.jbi.messaging.overSendSync";

    /**
     * Set the properties to the message exchange to specify that a synchronous
     * response is expected
     * 
     * @param exchange
     * @param overSyncExchange
     * 
     * @return {@code true} if the it is an 'overSync' exchange
     * 
     * @throws MessagingException
     *             The <code>sync_send()</code> is forbidden for acknowledgment
     *             send
     */
    public final static boolean setSendSyncProperties(final MessageExchangeImpl exchange)
            throws MessagingException {
        boolean result = false;

        if (!exchange.getStatus().equals(ExchangeStatus.ACTIVE)) {
            throw new MessagingException(
                    "Synchronous send is forbidden when sending an acknowledgement");
        }

        // specify the role of the requester of the sync exchange, used
        // internally
        if (MessageExchange.Role.PROVIDER.equals(exchange.getRole())) {
            if (exchange.getProperty(PROPERTY_SENDSYNC_PROVIDER) != null
                    && exchange.getProperty(PROPERTY_SENDSYNC_PROVIDER).equals(
                            Boolean.TRUE.toString())) {
                // set oversync flag
                exchange.setProperty(PROPERTY_OVERSENDSYNC, Boolean.TRUE.toString());
                result = true;
            }
            exchange.setProperty(PROPERTY_SENDSYNC_CONSUMER, Boolean.TRUE.toString());
        } else {
            if (exchange.getProperty(PROPERTY_SENDSYNC_CONSUMER) != null
                    && exchange.getProperty(PROPERTY_SENDSYNC_CONSUMER).equals(
                            Boolean.TRUE.toString())) {
                // set oversync flag
                exchange.setProperty(PROPERTY_OVERSENDSYNC, Boolean.TRUE.toString());
                result = true;
            }
            exchange.setProperty(PROPERTY_SENDSYNC_PROVIDER, Boolean.TRUE.toString());
        }

        // set the property as defined in the JBI specifications
        exchange.setProperty(DeliveryChannelImpl.PROPERTY_SENDSYNC, Boolean.TRUE.toString());

        return result;
    }

    /**
     * Update the sync properties of the message exchange
     * 
     * @param exchange
     * @return {@code true} if the it is an 'overSync' exchange
     */
    public final static boolean updateSyncProperties(final MessageExchangeImpl exchange) {
        boolean result = false;
        // remove the SENDSYNC property for eventually further steps of the
        // exchange
        if (exchange.getProperty(PROPERTY_OVERSENDSYNC) != null) {
            exchange.setProperty(PROPERTY_OVERSENDSYNC, null);
            result = true;
        } else {
            exchange.setProperty(DeliveryChannelImpl.PROPERTY_SENDSYNC, null);
        }
        if (MessageExchange.Role.PROVIDER.equals(exchange.getRole())) {
            exchange.setProperty(PROPERTY_SENDSYNC_PROVIDER, null);
        } else {
            exchange.setProperty(PROPERTY_SENDSYNC_CONSUMER, null);
        }

        return result;
    }

    /**
     * Return the synchronized mode related to a message exchange.
     * 
     * @param messageExchange
     *            the message exchange to check
     * @param isResponse
     *            {@code true} if it is a check for a response message, {@code
     *            false} otherwise
     * @return true if message is in CONSUMER role and it is a response or if
     *         message is in PROVIDER role and it isn't a response, {@code
     *         false} otherwise
     */
    public final static boolean getSyncMode(final MessageExchangeImpl messageExchange,
            final boolean isResponse) {
        final boolean syncMode;
        if ((MessageExchange.Role.CONSUMER.equals(messageExchange.getRole()) && isResponse)
                || (MessageExchange.Role.PROVIDER.equals(messageExchange.getRole()) && !isResponse)) {
            syncMode = Boolean.parseBoolean((String) messageExchange
                    .getProperty(TransporterUtil.PROPERTY_SENDSYNC_CONSUMER));
        } else {
            syncMode = Boolean.parseBoolean((String) messageExchange
                    .getProperty(TransporterUtil.PROPERTY_SENDSYNC_PROVIDER));
        }
        return syncMode;
    }
}
